@@ -73,12 +73,19 @@ async fn create_common<RT: Runtime, CT: ChanTarget>(
     rt: &RT,
     target: &CT,
 ) -> Result<PendingClientCirc> {
-    let chan = chanmgr.get_or_launch(target).await?;
+    let chan = chanmgr
+        .get_or_launch(target)
+        .await
+        .map_err(|cause| Error::Channel {
+            peer: OwnedChanTarget::from_chan_target(target),
+            cause,
+        })?;
     let (pending_circ, reactor) = chan.new_circ().await?;
 
     rt.spawn(async {
         let _ = reactor.run().await;
-    })?;
+    })
+    .map_err(|e| Error::from_spawn("circuit reactor task", e))?;
 
     Ok(pending_circ)
 }
@@ -289,13 +296,18 @@ impl<R: Runtime> CircuitBuilder<R> {
         self.path_config.replace(new_config);
     }
 
-    /// Flush state to the state manager.
-    pub(crate) fn save_state(&self) -> Result<()> {
+    /// Flush state to the state manager if we own the lock.
+    ///
+    /// Return `Ok(true)` if we saved, and `Ok(false)` if we didn't hold the lock.
+    pub(crate) fn save_state(&self) -> Result<bool> {
+        if !self.storage.can_store() {
+            return Ok(false);
+        }
         // TODO: someday we'll want to only do this if there is something
         // changed.
         self.builder.timeouts.save_state(&self.storage)?;
         self.guardmgr.store_persistent_state()?;
-        Ok(())
+        Ok(true)
     }
 
     /// Replace our state with a new owning state, assuming we have
@@ -384,10 +396,12 @@ where
     let inner_timeout_future = rt.timeout(abandon, fut);
     let outer_timeout_future = rt.timeout(timeout, rcv);
 
-    runtime.spawn(async move {
-        let result = inner_timeout_future.await;
-        let _ignore_cancelled_error = snd.send(result);
-    })?;
+    runtime
+        .spawn(async move {
+            let result = inner_timeout_future.await;
+            let _ignore_cancelled_error = snd.send(result);
+        })
+        .map_err(|e| Error::from_spawn("circuit construction task", e))?;
 
     let outcome = outer_timeout_future.await;
     // 4 layers of error to collapse:
@@ -399,7 +413,7 @@ where
     // (Technically, we could refrain from unwrapping the future's result,
     // but doing it this way helps make it more certain that we really are
     // collapsing all the layers into one.)
-    Ok(outcome????)
+    outcome???
 }
 
 #[cfg(test)]
